@@ -2,6 +2,7 @@
 
 import pprint
 import re
+from typing import Any, Dict, Optional
 
 from yea_wandb import backend
 
@@ -122,10 +123,10 @@ def parse_term(v, state, result=None):
     ind_list = []
     if len(spl) == 2:
         ind_list = re.findall(r"\[([^[\]]*)\]", "[" + spl[1])
-    found = state.get(var)
-    if found is None:
+    if var not in state:
         print("WARNING: Variable `{}` not found in state keys: {}".format(var, ",".join(state.keys())))
         return None
+    found = state.get(var)
     for ind in ind_list:
         if isinstance(found, list):
             ind = int(ind)
@@ -137,13 +138,13 @@ def parse_term(v, state, result=None):
                 return None
             lookup = found[ind]
         else:
+            if ind not in found:
+                bad = "key {} not found in {} [{}]".format(ind, var, ",".join(ind_list))
+                if result is not None:
+                    result.append("Not found: " + bad)
+                print("WARNING:", bad)
+                return None
             lookup = found.get(ind)
-        if lookup is None:
-            bad = "key {} not found in {} [{}]".format(ind, var, ",".join(ind_list))
-            if result is not None:
-                result.append("Not found: " + bad)
-            print("WARNING:", bad)
-            return None
         found = lookup
     return found
 
@@ -228,6 +229,14 @@ class YeaWandbPlugin:
     def monitors_start(self):
         self._backend.start()
 
+    def monitors_configure(self, config: Optional[Dict[str, Any]]):
+        if config is None:
+            return
+        # optionally configure backend (update context)
+        ctx = config.get("mock_server")
+        if ctx is not None:
+            self._backend.update_ctx(ctx)
+
     def monitors_stop(self):
         self._backend.stop()
 
@@ -275,6 +284,7 @@ class YeaWandbPlugin:
             runs.append(run)
 
         state[":wandb:artifacts"] = glob_parsed.artifacts
+        state[":wandb:sentry_events"] = glob_parsed.sentry_events
         state[":wandb:runs"] = runs
         # deprecate this
         state[":wandb:runs_len"] = len(runs)
